/* Copyright 2011-2012, Georgia Institute of Technology. All rights reserved. */
/*
 * Copyright (c) 1998-2008 GTARC. All Rights Reserved.
 *
 * This code was developed by Georgia Tech Research Institute under
 * contract with the Department of Justice.
 * Funding was provided by the Department of Justice (DOJ).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.gtri.icl.iead.policy_guide.wsc;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.soap.MTOMFeature;

import oasis.names.tc.xacml._2_0.context.schema.os.RequestType;
import oasis.names.tc.xacml._2_0.context.schema.os.ResponseType;

import org.gtri.icl.iead.policy_guide.arrest_exchange.ArrestReportType;
import org.gtri.icl.iead.policy_guide.arrest_extension.ArrestType;
import org.gtri.icl.iead.policy_guide.msg.ReadRequestType;
import org.gtri.icl.iead.policy_guide.msg.ReadResponseType;
import org.gtri.icl.iead.policy_guide.ws.FaultMsg;
import org.gtri.icl.iead.policy_guide.ws.SamplePortType;
import org.gtri.icl.iead.policy_guide.ws.SampleWebService;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.sun.xml.ws.api.security.trust.client.STSIssuedTokenConfiguration;
import com.sun.xml.ws.security.Token;
import com.sun.xml.ws.security.trust.GenericToken;
import com.sun.xml.ws.security.trust.STSIssuedTokenFeature;
import com.sun.xml.ws.security.trust.impl.client.DefaultSTSIssuedTokenConfiguration;

/**
 *
 * @author shrom
 */
public class SampleWSCClient {
    private static String wsdlUrl = "http://localhost:8080/m2wsp/services/samplews?wsdl";
    private static String sepUrl = "http://localhost:8080/m2wsp/services/samplews";
    
    private static Element createSamlAssertionType(File samlAssertionFile) {
		Element assertionElement = null;
    	try {
    		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    		DocumentBuilder db = dbf.newDocumentBuilder();
    		Document doc = db.parse(samlAssertionFile);
    		assertionElement = doc.getDocumentElement();
    	} catch(Exception ex) {}

		return assertionElement;
    }
    
    private static SamplePortType prepPort(Element samlAssertion) {
        SamplePortType port = null;
        SampleWebService webService = null;
        //Shows example of using server side precompiled WSDL file that contains correct end point url (assumption - make sure WSDL is available).        
        try {
            //Shows example of setting different location for WSDL and actual end point address for the web service
            webService = new SampleWebService(
                    new URL(wsdlUrl),
                    new QName("http://iead.icl.gtri.org/policy-guide/ws",
                    "SampleWebService"));
        } catch (MalformedURLException ex) {
            System.out.println("\n\n\n\nUnable to create port with specified WSDL URL, please verify WSDL URL\n\n\n\n");
            Logger.getLogger(SampleWSCClient.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        // -----------------------------STS stuff 
        STSIssuedTokenConfiguration config = new DefaultSTSIssuedTokenConfiguration();
//        DefaultSTSIssuedTokenConfiguration configd = new DefaultSTSIssuedTokenConfiguration();
	
        //Token actAsToken = getActAsToken(context);
        Token actAsToken = new GenericToken(samlAssertion);
//        config.getOtherOptions().put(STSIssuedTokenConfiguration.ACT_AS, actAsToken);
//        set OnBehalfOf token
        ((DefaultSTSIssuedTokenConfiguration) config).setOBOToken(actAsToken);
        STSIssuedTokenFeature sTSIssuedTokenFeature = new STSIssuedTokenFeature(config);
        MTOMFeature mtomFeature = new MTOMFeature(false);
        
        port = webService.getSamplePort(new WebServiceFeature[]{sTSIssuedTokenFeature, mtomFeature});
        
        ((BindingProvider) port).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, sepUrl);
        String urlUsed = (String) ((BindingProvider) port).getRequestContext().get(BindingProvider.ENDPOINT_ADDRESS_PROPERTY);
        System.out.println("Using Service End Point URL: " + urlUsed);
        
        return port;
    }
    
    private static void writeOutputFiles(ArrestReportType resultDocument, File xacmlRequestFile, File xacmlResponseFile, File dataFile)
    		throws JAXBException, IOException {

    	JAXBContext jc = JAXBContext.newInstance(ObjectFactoryFactory.ctxOF.getClass(),
    			ObjectFactoryFactory.policyOF.getClass(),
    			ObjectFactoryFactory.exchOF.getClass(),
    			ObjectFactoryFactory.extOF.getClass(),
    			ObjectFactoryFactory.aiOF.getClass(),
    			ObjectFactoryFactory.jxdmOF.getClass(),
    			ObjectFactoryFactory.niemOF.getClass(),
    			ObjectFactoryFactory.niemProxyOF.getClass(),
    			ObjectFactoryFactory.niemStructOF.getClass());

    	//write xacml request
        JAXBElement<RequestType> jaxbXacmlReq = resultDocument.getRequest();
    	JAXBContext jcReq = JAXBContext.newInstance(ObjectFactoryFactory.ctxOF.getClass());
        Marshaller mReq = jc.createMarshaller();
        mReq.marshal(jaxbXacmlReq, new FileOutputStream(xacmlRequestFile));
    	
    	//write xacml response
    	JAXBElement<ResponseType> jaxbXacmlResp = resultDocument.getResponse();
    	String respPkgNames = ObjectFactoryFactory.ctxOF.getClass().getPackage().getName();// +
    			//":" + ObjectFactoryFactory.policyOF.getClass().getPackage().getName();
    	JAXBContext jcResp = JAXBContext.newInstance(respPkgNames);
        Marshaller mResp = jc.createMarshaller();
        mResp.marshal(jaxbXacmlResp, new FileOutputStream(xacmlResponseFile));
    	
    	//write data
        JAXBElement<ArrestType> jaxbResult = resultDocument.getArrest();
        if(jaxbResult != null) {
        	String recordPkgNames = ObjectFactoryFactory.exchOF.getClass().getPackage().getName();// +
        			/*":" + ObjectFactoryFactory.extOF.getClass().getPackage().getName() +
        			":" + oasis.names.tc.xacml._2_0.context.schema.os.ObjectFactory.class.getPackage().getName() +
        			":" + oasis.names.tc.xacml._2_0.policy.schema.os.ObjectFactory.class.getPackage().getName() +
        			":" + arOF.getClass().getPackage().getName();*/
        	//JAXBContext jcData = JAXBContext.newInstance(recordPkgNames);
        	Marshaller mData = jc.createMarshaller();
        	mData.marshal(jaxbResult, new FileOutputStream(dataFile));
        } else {
        	/*FileWriter fw = new FileWriter(dataFile);
        	fw.write("[no data]");
        	fw.close();*/
        }
    }
    
    public static ReadResponseType read (Element samlAssertion, String recordId) {
        SamplePortType port = prepPort(samlAssertion);
        
        //TODO add schema validation through Service feature / include ErrorHandler from the server side to the library
//        WebServiceFeature feature = new SchemaValidationFeature(gov.niem.ws.sample.jaxwsspr.server.handler.ErrorHandler.class);


        //Document exchange
        ReadRequestType getDocumentRequestType = ObjectFactoryFactory.msgOF.createReadRequestType();
        JAXBElement<String> jaxbRecordId = ObjectFactoryFactory.msgOF.createRecordId(recordId);
        getDocumentRequestType.setRecordId(jaxbRecordId);
        
        ReadResponseType getDocumentResponseType = null;
		try {
			getDocumentResponseType = port.read(getDocumentRequestType);
		} catch (FaultMsg e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        System.out.println("Read finished");
        return getDocumentResponseType;
    }
    
    public static ReadResponseType executeRead(String recordId, File samlAssertionFile,
    		File requestFile, File responseFile, File dataFile) throws JAXBException, IOException {
    	
    	Element samlAssertion = createSamlAssertionType(samlAssertionFile);
    	ReadResponseType response = read(samlAssertion, recordId);

    	JAXBElement<ArrestReportType> jaxbResultDocument = response.getArrestReport();
    	ArrestReportType resultDocument = jaxbResultDocument.getValue();
    	writeOutputFiles(resultDocument, requestFile, responseFile, dataFile);

    	return response;
    }
    
    private static void printUsage() {
    	System.out.println("Usage:");
    	System.out.print(SampleWSCClient.class.getName());
    	System.out.println(" <record-id> <SAML Assertion File Path>" +
    			" <XACML Request File Path> <XACML Response File Path> <Data File Path>");
    	System.out.println("or");
    	System.out.println(SampleWSCClient.class.getName());
    	System.out.println("\tfor interactive mode");
    }
    
    public static void main(String[] args) throws Exception {
    	String recordId = null;
    	String samlAssertionFilePath = null;
    	String requestFilePath = null;
    	String responseFilePath = null;
    	String dataFilePath = null;

    	if(args.length == 0) {
    		BufferedReader inRdr = new BufferedReader(new InputStreamReader(System.in));
    		
    		System.out.println("Record ID");
    		recordId = inRdr.readLine();
    		
    		System.out.println("SAML Assertion File Path");
    		samlAssertionFilePath = inRdr.readLine();
    		
    		System.out.println("XACML Request File Path");
    		requestFilePath = inRdr.readLine();
    		
    		System.out.println("XACML Response File Path");
    		responseFilePath = inRdr.readLine();
    		
    		System.out.println("Data File Path");
    		dataFilePath = inRdr.readLine();
    	} else if(args.length == 5) {
    		recordId = args[0];
    		samlAssertionFilePath = args[1];
    		requestFilePath = args[2];
    		responseFilePath = args[3];
    		dataFilePath = args[4];
    	} else {
    		printUsage();
    		System.exit(-1);
    	}
    	
    	File samlAssertionFile = new File(samlAssertionFilePath);
    	File requestFile = new File(requestFilePath);
    	File responseFile = new File(responseFilePath);
    	File dataFile = new File(dataFilePath);
    	
    	SampleWSCClient.executeRead(recordId, samlAssertionFile, requestFile, responseFile, dataFile);
    }
}
