/* Copyright 2011-2012, Georgia Institute of Technology. All rights reserved. */
package org.gtri.icl.iead.policy_guide.service;

import java.io.InputStream;

import javax.servlet.ServletContext;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.gtri.icl.iead.policy_guide.arrest_extension.ArrestType;

/**
 * The DataAdapter is used to read a sample arrest data record from disk 
 * and marshal it into a JAXBElement&lt;ArrestType&gt; object. 
 * @author Erin.Reddick AT gtri.gatech.edu
 *
 */
public class DataAdapter {

	/**
	 * The path to the directory containing the records.
	 */
	private final String dataDirectoryPath;
	
	/**
	 * Basic constructor setting the dataDirectoryPath
	 * @param dataDirectoryPath path to the directory containing the records
	 */
	public DataAdapter(String dataDirectoryPath) {
		this.dataDirectoryPath = dataDirectoryPath;
	}
	
	/**
	 * Uses servletContext to read the arrest record identified by recordId.
	 * Returns the JAXB ArrestType representation.
	 * @param servletContext The getResourceAsStream method is used to
	 * read the record
	 * @param recordId identifier for the record to read
	 * @return JAXB ArrestType representation of the record identified by recordId
	 * @throws JAXBException on a unmarshalling error
	 */
	public ArrestType retrieveRecord(ServletContext servletContext, String recordId) throws JAXBException {
		InputStream is = servletContext.getResourceAsStream(dataDirectoryPath + "/" + recordId + ".xml");

		if(is == null)
			return null;
		
		String pkgList = ObjectFactoryFactory.extOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.ctxOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.policyOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.aiOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.jxdmOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.fipsOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.niemOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.niemProxyOF.getClass().getPackage().getName() +
				":" + ObjectFactoryFactory.niemStructOF.getClass().getPackage().getName();

		JAXBContext jc = JAXBContext.newInstance(pkgList);
		Unmarshaller u = jc.createUnmarshaller();

		@SuppressWarnings("unchecked")
		JAXBElement<ArrestType> jaxbAr = (JAXBElement<ArrestType>)u.unmarshal(is);

		return jaxbAr.getValue();
	}
}
