/* Copyright 2011-2012, Georgia Institute of Technology. All rights reserved. */
package org.gtri.icl.iead.policy_guide.test;

import java.io.File;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import oasis.names.tc.xacml._2_0.context.schema.os.ResponseType;
import oasis.names.tc.xacml._2_0.context.schema.os.ResultType;
import oasis.names.tc.xacml._2_0.policy.schema.os.ObligationType;
import oasis.names.tc.xacml._2_0.policy.schema.os.ObligationsType;

import org.gtri.icl.iead.policy_guide.arrest_exchange.ArrestReportType;
import org.gtri.icl.iead.policy_guide.arrest_extension.ArrestType;
import org.gtri.icl.iead.policy_guide.msg.ReadRequestType;
import org.gtri.icl.iead.policy_guide.msg.ReadResponseType;
import org.gtri.icl.iead.policy_guide.service.ObjectFactoryFactory;
import org.junit.Assert;
import org.junit.Test;

import com.sun.xml.wss.saml.internal.saml20.jaxb20.AssertionType;

public class TestSuite {

	public static final String BASE_PATH = "src" + File.separator + "test";
	public static final String META_INF_PATH = BASE_PATH + File.separator + "META-INF";
	public static final String SUNXACML_CONFIG_PATH = File.separator + "META-INF" + File.separator + "policies" + File.separator + "SunXACMLConfig.xml";
	public static final String OBLIGATION_HANDLER_CONFIG_PATH = File.separator + "META-INF" + File.separator + "ObligationHandlerConfig.xml";
	public static final String GFIPM_USER_FILE_PATH = File.separator + "META-INF" + File.separator + "gfipm-metadata-2.0-user.txt";
	public static final String DATA_DIRECTORY_PATH = File.separator + "META-INF" + File.separator + "records";
	public static final String[] EXPECTED_OBLIGATION_IDS = new String[]{"LogValidAccess", "NotifyViaEmail"};
	
	private TestWebServiceImpl wsImpl;

	@Test
	public void testCase1() throws Exception {
		String assertionFilename = "saml_assertion_user01.xml";
		String recordId = "Record-1";
		testCaseHelper(assertionFilename, recordId, "Deny", new String[]{});
	}

	@Test
	public void testCase2() throws Exception {
		String assertionFilename = "saml_assertion_user02.xml";
		String recordId = "Record-2";
		testCaseHelper(assertionFilename, recordId, "Permit", new String[]{});
	}

	@Test
	public void testCase3() throws Exception {
		String assertionFilename = "saml_assertion_user03.xml";
		String recordId = "Record-2";
		testCaseHelper(assertionFilename, recordId, "Permit", new String[]{});
	}

	@Test
	public void testCase4() throws Exception {
		String assertionFilename = "saml_assertion_user04.xml";
		String recordId = "Record-1";
		testCaseHelper(assertionFilename, recordId, "Deny", new String[]{});
	}

	@Test
	public void testCase5() throws Exception {
		String assertionFilename = "saml_assertion_user05.xml";
		String recordId = "Record-1";
		testCaseHelper(assertionFilename, recordId, "Deny", new String[]{});
	}

	@Test
	public void testCase6() throws Exception {
		String assertionFilename = "saml_assertion_user05.xml";
		String recordId = "Record-2";
		testCaseHelper(assertionFilename, recordId, "Permit", EXPECTED_OBLIGATION_IDS);
	}
	
	private void testCaseHelper(String assertionFilename, String recordId, String expectedDecision, String[] expectedObligationIds) throws Exception {
		//initialize test web service impl
		JAXBElement<AssertionType> samlAssertion = readSamlAssertion(assertionFilename);
		wsImpl = new TestWebServiceImpl(samlAssertion, BASE_PATH, SUNXACML_CONFIG_PATH, OBLIGATION_HANDLER_CONFIG_PATH,
				GFIPM_USER_FILE_PATH, DATA_DIRECTORY_PATH);
		
		//build service request
		ReadRequestType readRequest = new ReadRequestType();
		readRequest.setRecordId(ObjectFactoryFactory.msgOF.createRecordId(recordId));
		
		//invoke test service impl and get service response
		ReadResponseType readResponse = wsImpl.read(readRequest);
		
		//assert properties of the service response
		//XACML decision should be Permit
		//Obligations returned should be: NotifyViaEmail and LogValidAccess
		//Record-2 should be returned

		Assert.assertNotNull(readResponse);
		
		ArrestReportType arrestReport = readResponse.getArrestReport().getValue();
		ResponseType xacmlResponse = arrestReport.getResponse().getValue();
		ResultType xacmlResult = xacmlResponse.getResult().get(0);
		Assert.assertEquals("XACML Decision", expectedDecision, xacmlResult.getDecision().value());
		
		ObligationsType obligations = xacmlResult.getObligations();
		if(obligations != null) {
			String[] actualObligationIds = new String[2];
			for(ObligationType obligation : obligations.getObligation()) {
				String obligationId = obligation.getObligationId();
				if(obligation.getObligationId().equals("NotifyViaEmail"))
					actualObligationIds[1] = obligationId;
				else if(obligation.getObligationId().equals("LogValidAccess"))
					actualObligationIds[0] = obligationId;
				else
					throw new RuntimeException("Unknown obligation received: " + obligationId);
			}
			Assert.assertArrayEquals(expectedObligationIds, actualObligationIds);
		} else {
			Assert.assertArrayEquals(expectedObligationIds, new String[]{});
		}

		JAXBElement<ArrestType> jaxbArrest = arrestReport.getArrest();
		if(expectedDecision.equals("Permit")) {
			ArrestType arrest = jaxbArrest.getValue();
			String actualRecordId = arrest.getId();
			Assert.assertEquals(recordId, actualRecordId);
		} else {
			Assert.assertNull(jaxbArrest);
		}
	}
	
	@SuppressWarnings("unchecked")
	private JAXBElement<AssertionType> readSamlAssertion(String assertionFilename) throws JAXBException {
		File samlAssertionFile = new File(BASE_PATH, assertionFilename);
		JAXBContext jc = JAXBContext.newInstance(AssertionType.class);
		Unmarshaller u = jc.createUnmarshaller();
		return (JAXBElement<AssertionType>)u.unmarshal(samlAssertionFile);
	}
	
	public static void main(String[] args) throws Exception {
		new TestSuite().testCase1();
	}
}
